import logging
from logging import Logger


def set_context(logger, value):
    """
    Walks the tree of loggers and tries to set the context for each handler

    :param logger: logger
    :param value: value to set
    """
    _logger = logger
    while _logger:
        for handler in _logger.handlers:
            try:
                handler.set_context(value)
            except AttributeError:
                # Not all handlers need to have context passed in so we ignore
                # the error when handlers do not have set_context defined.
                pass
        if _logger.propagate is True:
            _logger = _logger.parent
        else:
            _logger = None


class LoggingMixin:
    """Convenience super-class to have a logger configured with the class name"""

    def __init__(self, context=None):
        self._set_context(context)

    @property
    def log(self) -> Logger:
        """Returns a logger."""
        try:
            # FIXME: LoggingMixin should have a default _log field.
            return self._log
        except AttributeError:
            self._log = logging.getLogger(self.__class__.__module__ + '.' + self.__class__.__name__)
            return self._log

    def _set_context(self, context):
        if context is not None:
            set_context(self.log, context)



from setuptools import setup, find_packages


def readme():
    with open('README.md', 'r') as f:
        return f.read()


setup(
    name='sentry-io',
    version='0.0.1',
    description='',
    long_description=readme(),
    long_description_content_type='text/markdown',
    url='https://github.com/AhmadLabs/Architect',
    license='MIT',
    author='Hamza Ahmad',
    author_email='hamza.ahmad@me.com',
    packages=find_packages(),
    install_requires=[
        'cached-property~=1.5',
        'dacite',
        'pydantic',
    ],
    extras_require={'dev': []},
    include_package_data=True,
    zip_safe=False,
    classifiers=[],
    keywords='python',
)



from abc import ABC, abstractmethod
from typing import Optional, Any, Dict

from cached_property import cached_property

from ontelligence.core.backends import SecretsBackend, ConnectionsBackend
from architect.core.logging import LoggingMixin


class BaseProvider(LoggingMixin):

    __secret_schema = None
    __connection_schema = None

    def __init__(self, secret_schema=None, connection_schema=None):
        super().__init__()
        self.__secret_schema = secret_schema
        self.__connection_schema = connection_schema

    def _get_connection(self, conn_id, override_data: Optional[Dict[str, Any]] = None):
        if not self.__connection_schema:
            raise ValueError('Please provide connection schema')
        self.log.info(f'Retrieving connection key: "{conn_id}"')
        return ConnectionsBackend().get_backend().parse_key(data_class=self.__connection_schema, key=conn_id, override_data=override_data)

    def _get_secret(self, secret_id, override_data: Optional[Dict[str, Any]] = None):
        if not self.__secret_schema:
            raise ValueError('Please provide secret schema')
        self.log.info(f'Retrieving secret key: "{secret_id}"')
        return SecretsBackend().get_backend().parse_key(data_class=self.__secret_schema, key=secret_id, override_data=override_data)

    @cached_property
    def conn(self):
        raise NotImplementedError

    def get_conn(self):
        return self.conn
